
/* aft_fe_test_te1.c */

#include "aft_fe_test_te1.h"

#if defined(__WINDOWS__)
/*# pragma comment( lib, "libsangoma" )*/
# define _CRT_SECURE_NO_WARNINGS
#endif

#include "libsangoma.h"

#if defined(__LINUX__)
# include <linux/if_packet.h> /* sockaddr_ll */
#endif



/******************************************************************************
**			   DEFINES and MACROS
******************************************************************************/

#define FE_INFO		printf
#define FE_DEBUG	if(0)printf
#define FE_ERROR	printf("Error: %s(): line:%d\n", __FUNCTION__, __LINE__);printf	

/* Maxim chip definitions */
#define REG_IDR			0xF8
#define DEVICE_ID_DS26528	0x0B
#define DEVICE_ID_DS26524	0x0C
#define DEVICE_ID_DS26522	0x0D
#define DEVICE_ID_DS26521	0x0E
#define DEVICE_ID_BAD		0x00

#define DEVICE_ID_SHIFT		3
#define DEVICE_ID_MASK		0x1F
#define DEVICE_ID_DS(dev_id)	((dev_id) >> DEVICE_ID_SHIFT) & DEVICE_ID_MASK
#define DECODE_CHIPID(chip_id)					\
		(chip_id == DEVICE_ID_DS26528) ? "DS26528" :	\
		(chip_id == DEVICE_ID_DS26524) ? "DS26524" :	\
		(chip_id == DEVICE_ID_DS26522) ? "DS26522" :	\
		(chip_id == DEVICE_ID_DS26521) ? "DS26521" : "Unknown"

#define FE_TEST_ACTION_READ		1
#define FE_TEST_ACTION_WRITE	2

/******************************************************************************
**			STRUCTURES AND TYPEDEFS
******************************************************************************/



/******************************************************************************
**			   GLOBAL VARIABLES
******************************************************************************/
static int fe_id = 0x00;

/*! Sangoma IO device descriptor */
static sng_fd_t sangoma_dev;
/*! Sangoma device name */
static char ifname[FNAME_LEN];

/* variable initialized by command-line parser */
static int wanpipe_number, interface_number, action, rw_register, w_data;

/******************************************************************************
**			  FUNCTION PROTOTYPES
******************************************************************************/


/******************************************************************************
**			  FUNCTION DEFINITIONS
******************************************************************************/

/*===========================================================================*/

/* SIOC_WAN_DEVEL_IOCTL not in libsangoma, yet */
int wan_aftup_ioctl(sng_fd_t fd, wan_cmd_api_t *api_cmd, int cmd)
{
#if defined(__WINDOWS__)

	DWORD			ln;
    unsigned char	id = 0;
	int				err = 0;
	wan_udp_hdr_t	wan_udp;

	memset(&wan_udp, 0x00, sizeof(wan_udp));

	wan_udp.wan_udphdr_request_reply = 0x01;
	wan_udp.wan_udphdr_id = id;
	wan_udp.wan_udphdr_return_code = WAN_UDP_TIMEOUT_CMD;

	wan_udp.wan_udphdr_command	= SIOC_WAN_DEVEL_IOCTL;
	wan_udp.wan_udphdr_data_len	= sizeof(wan_cmd_api_t);

	api_cmd->cmd = cmd;
	/* copy data from caller's buffer to driver's buffer */
	memcpy( wan_udp.wan_udphdr_data, 
			(void*)api_cmd,
			sizeof(wan_cmd_api_t));

	if(DeviceIoControl(
			fd,
			IoctlManagementCommand,
			(LPVOID)&wan_udp,
			sizeof(wan_udp_hdr_t),
			(LPVOID)&wan_udp,
			sizeof(wan_udp_hdr_t),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
			) == FALSE){
		err = 1;
	}else{
		err = 0;
	}

	if(err){
		FE_ERROR("wan_aftup_ioctl(): DeviceIoControl failed!!\n");
		return 1;
	}

	if(wan_udp.wan_udphdr_return_code != WAN_CMD_OK){
		FE_ERROR("wan_aftup_ioctl(): command failed! Return code: 0x%X.\n",
			wan_udp.wan_udphdr_return_code);
		return 2;
	}

	/* copy data from driver to caller's buffer */
	memcpy( (void*)api_cmd,
			wan_udp.wan_udphdr_data, 
			sizeof(wan_cmd_api_t));
	return 0;

#else

	int ret;
	struct ifreq req;

	memset(&req, 0x00, sizeof(req));

	api_cmd->cmd = cmd;
	req.ifr_data = (void*)api_cmd;
	strncpy(req.ifr_name, ifname, strlen(ifname));
	if ((ret = ioctl(sangoma_dev, SIOC_WAN_DEVEL_IOCTL, &req)) < 0){
		FE_ERROR("wan_aftup_ioctl(): ioctl() failed! Return code: 0x%X.\n",
			ret);
		return -EINVAL;
	}
	return 0;
#endif
}

/*===========================================================================*/

int exec_read_cmd(sng_fd_t fd, unsigned int off, unsigned int len, unsigned int *data)
{
	int err;
	wan_cmd_api_t api_cmd;

	api_cmd.cmd	= SIOC_WAN_READ_REG;
	api_cmd.bar	= 0;
	api_cmd.len	= len;
	api_cmd.offset	= off;
	
	err = wan_aftup_ioctl(fd, &api_cmd, api_cmd.cmd);
	if (err){
		FE_ERROR("wan_aftup_ioctl() failed!\n");
	}

	if (len==1){
		*(unsigned char*)data = *(unsigned char*)api_cmd.data;
	}else if (len==2){
		*(unsigned short*)data = *(unsigned short*)api_cmd.data;
	}else{
		*(unsigned int*)data = *(unsigned int*)api_cmd.data;
	}

	return err;

}

int exec_write_cmd(sng_fd_t fd, unsigned int off, unsigned int len, unsigned int data)
{
	int err;
	wan_cmd_api_t api_cmd;

	api_cmd.cmd	= SIOC_WAN_WRITE_REG;
	api_cmd.bar	= 0;
	api_cmd.len	= len;
	api_cmd.offset	= off;

	if (len==1){
		*(unsigned char*)api_cmd.data=(unsigned char)data;
	}else if (len==2){
		*(unsigned short*)api_cmd.data=(unsigned short)data;
	}else{
		*(unsigned int*)api_cmd.data=(unsigned int)data;
	}

	err = wan_aftup_ioctl(fd, &api_cmd, api_cmd.cmd);
	if (err){
		FE_ERROR("wan_aftup_ioctl() failed!\n");
	}

	return err;
}

/*===========================================================================*/

unsigned int write_cpld(unsigned short cpld_off,unsigned short cpld_data)
{
	cpld_off &= ~AFT8_BIT_DEV_ADDR_CLEAR; 
	cpld_off |= AFT8_BIT_DEV_ADDR_CPLD; 

	exec_write_cmd(sangoma_dev, 0x46, 2, cpld_off);
	exec_write_cmd(sangoma_dev, 0x44, 2, cpld_data);
	return 0;
}
#if 0
unsigned int read_cpld(unsigned short cpld_off)
{
	unsigned int cpld_data;

	cpld_off &= ~AFT8_BIT_DEV_ADDR_CLEAR; 
	cpld_off |= AFT8_BIT_DEV_ADDR_CPLD; 

	exec_write_cmd(sangoma_dev, 0x46, 2, cpld_off);

	if (exec_read_cmd(sangoma_dev, 0x44, 4, &cpld_data) == 0){
		return cpld_data;
	}else{
		return 0;
	}
}
#endif
/*===========================================================================*/

static int fe_test_disable_global_chip_reset()
{
	unsigned int data;
	int err, i;

	for (i = 0; i < 2; i++) {
		/* put in reset */
		write_cpld(0x00, 0x06);	

		sangoma_msleep(50);

		/* Disable Global Chip/FrontEnd/CPLD Reset */
		err = exec_read_cmd(sangoma_dev, 0x40, 4, &data);
		if (err){
			return 1;
		}

		data &= ~0x6;

		err = exec_write_cmd(sangoma_dev, 0x40, 4, data);
		if (err){
			return 1;
		}
	
		sangoma_msleep(50);
	}

	return 0;
}

static sng_fd_t fe_test_open_dev_by_name(const char *dev_name)
{
#if defined(__WINDOWS__)

	_snprintf(ifname, sizeof(ifname), "\\\\.\\%s", dev_name);

	printf("%s(): dev_name: %s\n", __FUNCTION__, dev_name);

	return CreateFile(	ifname, 
						GENERIC_READ | GENERIC_WRITE, 
						FILE_SHARE_READ | FILE_SHARE_WRITE,
						(LPSECURITY_ATTRIBUTES)NULL, 
						OPEN_EXISTING,
						FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH,
						(HANDLE)NULL
						);
#elif defined(__LINUX__)
	
	char error_msg[100];
	struct ifreq		ifr;
	struct sockaddr_ll	sa;
	
	_snprintf(ifname, sizeof(ifname), "%s", dev_name);

	memset(&sa,0,sizeof(struct sockaddr_ll));
	memset(&ifr,0,sizeof(struct ifreq));
	errno = 0;
   	sangoma_dev = socket(AF_PACKET, SOCK_DGRAM, 0);
   	if( sangoma_dev < 0 ) {
      		perror("Socket");
      		return INVALID_HANDLE_VALUE;
   	} /* if */
  
	strncpy(ifr.ifr_name, dev_name, strlen(dev_name));
	if (ioctl(sangoma_dev,SIOCGIFINDEX,&ifr)){
		snprintf(error_msg, 100, "Get index: %s", dev_name);
		perror(error_msg);
		close(sangoma_dev);
      	return INVALID_HANDLE_VALUE;
	}

	sa.sll_protocol = htons(0x17);
	sa.sll_family=AF_PACKET;
	sa.sll_ifindex = ifr.ifr_ifindex;

 	if(bind(sangoma_dev, (struct sockaddr *)&sa, sizeof(struct sockaddr_ll)) < 0){
		snprintf(error_msg, 100, "Bind: %s", dev_name);
    	perror(error_msg);
		close(sangoma_dev);
      	return INVALID_HANDLE_VALUE;
	}

	return sangoma_dev;
#else
# error "Unsupported OS"
#endif
}

/* no checks done for multiple open */
static sng_fd_t fe_test_open_api_span_chan(int span, int chan)
{
	char tmp_ifname[100];

#if defined(__WINDOWS__)
	/* Form the Interface Name from span and chan number (i.e. wanpipe1_if1). */
	_snprintf(tmp_ifname, sizeof(tmp_ifname), WP_INTERFACE_NAME_FORM, span, chan);
#elif defined(__LINUX__)
	/* Form the Interface Name from span and chan number (i.e. w1g1). */
	_snprintf(tmp_ifname, sizeof(tmp_ifname), "w%dg%d", span, chan);
#else
# error "Unsupported OS"
#endif
	return fe_test_open_dev_by_name(tmp_ifname);
}            

static int fe_test_open_api(int span, int chan) 
{
	sangoma_dev = fe_test_open_api_span_chan(span, chan);

    if (sangoma_dev == INVALID_HANDLE_VALUE){
		FE_ERROR("Unable to open Span %d Chan %d for general commands!\n", 
			span, chan);
		return 1;
	}

	FE_DEBUG("Successfuly opened Span %d Chan %d\n", span, chan);

	return 0;
}

static int check_id_reg()
{
	int	val;

	val = TE1_READ_REG( REG_IDR );

	FE_DEBUG("val = 0x%X\n", val);
	
	fe_id = DEVICE_ID_DS(val);

	switch(fe_id)
	{
	case DEVICE_ID_DS26528:
		FE_DEBUG("Device is DS26528!\n");
		break;
	case DEVICE_ID_DS26524:
		FE_DEBUG("Device is DS26524!\n");
		break;
/*
	case DEVICE_ID_DS26522:	//not installed on Sangoma HW
		FE_INFO("Device is DS26522!\n");
		break;	
*/
	case DEVICE_ID_DS26521:
		FE_DEBUG("Device is DS26521!\n");
		break;	
	default:
		FE_ERROR("Failed to read Maxim ID (%02X)\n", val);
		return 1;
	}

	return 0;
}


/*==========================================================================*/

static int __parse_command_line_args(int argc, char* argv[])
{
	int i;

	wanpipe_number = -1;
	interface_number = -1;
	action = -1;
	rw_register = -1;
	w_data = -1;

	for(i = 1; i < argc;){
	
		if(_stricmp(argv[i], "-c") == 0){
			if (i+1 > argc-1){
				FE_ERROR("No Span number was provided!\n");
				return 1;
			}
			wanpipe_number = (uint16_t)atoi(argv[i+1]);
			FE_DEBUG("Using Span number %d\n", wanpipe_number);
			i++;

		}else if(_stricmp(argv[i], "-i") == 0){
			if (i+1 > argc-1){
				FE_ERROR("No Channel number was provided!\n");
				return 1;
			}
			interface_number = (uint16_t)atoi(argv[i+1]);
			FE_DEBUG("Using Channel number %d\n", interface_number);
#if defined(__WINDOWS__)
			/* interface numbers are zero-based */
			interface_number -= 1;
#endif
			i++;

		}else if(_stricmp(argv[i], "-r") == 0){

			action = FE_TEST_ACTION_READ;

			if (i+1 > argc-1){
				FE_ERROR("No Read Register number was provided!\n");
				return 1;
			}
			sscanf(argv[i+1], "%x", &rw_register);
			FE_DEBUG("Seting Read Register to: 0x%08X\n", rw_register);
			i++;

		}else if(_stricmp(argv[i], "-w") == 0){

			action = FE_TEST_ACTION_WRITE;
			if (i+1 > argc-1){
				FE_ERROR("No Write Register number was provided!\n");
				return 1;
			}
			sscanf(argv[i+1], "%x", &rw_register);
			FE_DEBUG("Setting Write Register: 0x%08X\n", rw_register);
			i++;

		}else if(_stricmp(argv[i], "-d") == 0){
			if (i+1 > argc-1){
				FE_ERROR("No Write Data was provided!\n");
				return 1;
			}
			sscanf(argv[i+1], "%x", &w_data);
			FE_DEBUG("Using Write Data to: 0x%08X\n", w_data);
			i++;

		}else{
			FE_ERROR("Invalid Argument '%s'\n", argv[i]);
			return 1;
		}

		i++;
	}/* for() */

	if (action < 0) {
		FE_ERROR("Action not in command-line!\n");
		return 1;
	}

	if (rw_register < 0) {
		FE_ERROR("Register not in command-line!\n");
		return 1;
	}

	if (action == FE_TEST_ACTION_WRITE && w_data < 0) {
		FE_ERROR("No Data for Write Action in command-line!\n");		
		return 1;
	}

	return 0;
}

static const char *USAGE_STR =
"\n"
"Usage: aft_fe_test_te1 <-c> <-i> <-r|-w> <hex register> [-d <data for -w option> ]\n"
"\n"
"\t-c	Span/Card number: 1,2,3...\n"
"\t-i	Interface number 1,2,3,....\n"
"options:\n"
"\t-r	Read Register\n"
"\t-w	Write Register\n"
"\t-d	Data to Write for -w option\n"
"\n"
"Example 1: read Chip ID\n"
"\t aft_fe_test_te1 -c 1 -i 1 -r F8\n"
"Example 2: clear Chip reset\n"
"\t aft_fe_test_te1 -c 1 -i 1 -w F5 -d 0\n";

static int parse_command_line_args(int argc, char* argv[])
{
	int rc;

	rc = __parse_command_line_args(argc, argv);

	if (rc) {
		FE_INFO(USAGE_STR);
	}

	return rc;
}

/*===========================================================================*/
/*         Public functions which can be called by outside code.             */
/*===========================================================================*/

unsigned int write_maxim(unsigned short reg, unsigned short data)
{
	unsigned int tmp;

	if (reg & 0x800)  reg |= 0x2000;
	if (reg & 0x1000) reg |= 0x4000;
	
	reg &= ~AFT8_BIT_DEV_ADDR_CLEAR; 
	//reg |= AFT8_BIT_DEV_MAXIM_ADDR_CPLD;

	FE_DEBUG("%s(): Reg 0x%04X Data 0x%04X\n", __FUNCTION__, reg, data);

	/* write the register where the data will be written to*/
	exec_write_cmd(sangoma_dev, AFT_MCPU_INTERFACE_ADDR, 2, reg);

	/* read to avoid bridge optimization of two writes by PCI */
	exec_read_cmd(sangoma_dev, AFT_MCPU_INTERFACE, 4, &tmp);

	/* write the data to the register */
	exec_write_cmd(sangoma_dev, AFT_MCPU_INTERFACE, 2, data);

	return 0;
}

unsigned int read_maxim(unsigned short reg)
{
	unsigned int data;
	
	if (reg & 0x800)  reg |= 0x2000;
	if (reg & 0x1000) reg |= 0x4000;

	reg &= ~AFT8_BIT_DEV_ADDR_CLEAR; 
	//reg |= AFT8_BIT_DEV_MAXIM_ADDR_CPLD;

	/* write the register where data will be read from */
	if (exec_write_cmd(sangoma_dev, AFT_MCPU_INTERFACE_ADDR, 2, reg)) {
		FE_ERROR("Failed to write to AFT_MCPU_INTERFACE_ADDR!\n");
		return 0;
	}

	if (exec_read_cmd(sangoma_dev, AFT_MCPU_INTERFACE, 4, &data)) {
		FE_ERROR("Failed to read from AFT_MCPU_INTERFACE!\n");
		return 0;
	}

	FE_DEBUG("%s(): Reg 0x%04X Data 0x%08X\n", __FUNCTION__, reg, data);

	return (unsigned char)data;
}

/*********************************************************************/
/* call fe_test_global_init() only once - it is a global initialization */
/*********************************************************************/
int fe_test_global_init(int span, int chan)
{
	if (fe_test_open_api(span, chan)) {
		return 1;
	}
	
#if 1
	if (fe_test_disable_global_chip_reset()) {
		return 1;
	}
#endif

	/* read the Chip ID and check is it valid */
	return check_id_reg();
}

/*===========================================================================*/

int main(int argc, char *argv[])
{
	unsigned char rval;

	if (parse_command_line_args(argc, argv)) {
		return 1;
	}

	if (fe_test_global_init(wanpipe_number, interface_number)) {
		return 1;
	}

	switch(action)
	{
	case FE_TEST_ACTION_READ:
		rval = TE1_READ_REG(rw_register);
		FE_INFO("Read  [0x%04X]: 0x%02X\n", rw_register, rval);
		break;
	case FE_TEST_ACTION_WRITE: 
		FE_INFO("Write [0x%04X]: 0x%02X\n", rw_register, w_data);
		TE1_WRITE_REG(rw_register, w_data);
		break;
	}
	return 0;
}

/*===========================================================================*/
